package com.luojigou.xiaodou

import android.annotation.SuppressLint
import android.app.Service
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothManager
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.IBinder
import android.util.Log
import androidx.core.app.NotificationChannelCompat
import androidx.core.app.NotificationCompat
import androidx.core.app.NotificationManagerCompat
import com.luojigou.xiaodou.ble.XDScanBLEDevice
import java.util.UUID
import kotlin.concurrent.thread


@SuppressLint("MissingPermission")
class XDScanBLEService : Service() {
    companion object {
        const val ACTION_START_SCAN = "com.luojigou.xiaodou.ble.ACTION_START_SCAN"
        const val ACTION_STOP_SCAN = "com.luojigou.xiaodou.ble.ACTION_STOP_SCAN"

        const val ACTION_SCAN_RESULT = "com.luojigou.xiaodou.ble.ACTION_SCAN_RESULT"
        const val ACTION_SCAN_RESULT_DATA = "com.luojigou.xiaodou.ble.ACTION_SCAN_RESULT_DATA"

        const val ACTION_CONNECT_WIFI = "com.luojigou.xiaodou.ble.ACTION_CONNECT_WIFI"
        const val ACTION_CONNECT_WIFI_RESULT_ERROR = "com.luojigou.xiaodou.ble.ACTION_CONNECT_WIFI_RESULT_ERROR"
        const val ACTION_CONNECT_WIFI_RESULT_SUCCESS = "com.luojigou.xiaodou.ble.ACTION_CONNECT_WIFI_RESULT_SUCCESS"
        const val ACTION_CONNECT_WIFI_RESULT_FAILED = "com.luojigou.xiaodou.ble.ACTION_CONNECT_WIFI_RESULT_FAILED"

        private val wifiConnectServiceUUID = UUID.fromString("0000ae80-0000-1000-8000-00805f9b34fb")
        private val wifiConnectWriteUUID = UUID.fromString("0000ae81-0000-1000-8000-00805f9b34fb")
        private val wifiConnectNotifyUUID = UUID.fromString("0000ae82-0000-1000-8000-00805f9b34fb")

        private val bleChannelName = "BLE-CHANNEL"
    }

    private val deviceMap: MutableMap<XDScanBLEDevice, BluetoothDevice> = HashMap()

    private lateinit var bluetoothManager: BluetoothManager

    private lateinit var bluetoothAdapter: BluetoothAdapter

    private lateinit var bluetoothReceiver: BroadcastReceiver

    @Volatile
    private var bleScanning = false

    private val bleScanSettings = ScanSettings.Builder()
        .setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY)
        .build()

    private val bleScanFilters = mutableListOf<ScanFilter>()

    private val bleScanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult) {
            super.onScanResult(callbackType, result)
            val oldSize = deviceMap.size
            addDevice(result)
            val newSize = deviceMap.size

            if (oldSize != newSize) {
                sendResult()
            }
        }

        override fun onBatchScanResults(results: MutableList<ScanResult>) {
            super.onBatchScanResults(results)
            val oldSize = deviceMap.size
            results.forEach { addDevice(it) }
            val newSize = deviceMap.size

            if (oldSize != newSize) {
                sendResult()
            }

        }

        override fun onScanFailed(errorCode: Int) {
            super.onScanFailed(errorCode)
            Log.d("XDScanBLEService", "onScanFailed: $errorCode")
        }

        private fun addDevice(result: ScanResult) {
            result.apply {
                if (device.name == null) return

                val device = XDScanBLEDevice(
                    name = device.name ?: "unknown",
                    bondState = device.bondState.toString(),
                    rssi = rssi.toString(),
                    address = device.address,
                    addressType = device.type.toString(),
                )

                if (!deviceMap.contains(device)) {
                    deviceMap[device] = result.device
                }
            }

        }
    }

    override fun onCreate() {
        super.onCreate()
        goForeground()
        initBluetooth()
    }

    private fun goForeground() {
        val manager = NotificationManagerCompat.from(this)
        val notificationChannel = NotificationChannelCompat.Builder(
            bleChannelName, NotificationManagerCompat.IMPORTANCE_LOW
        ).setName("蓝牙扫描通知").setDescription("用于蓝牙扫描硬件设备并配网处理").build()

        if (manager.areNotificationsEnabled()) {
            if (manager.getNotificationChannel(bleChannelName) == null) {
                manager.createNotificationChannel(notificationChannel)
            }

            val notification = NotificationCompat.Builder(this, bleChannelName)
                .setContentTitle("XiaoDou")
                .setContentText("XiaoDou is running")
                .setSmallIcon(android.R.drawable.stat_sys_data_bluetooth)
                .setChannelId(bleChannelName)
                .build()

            startForeground(1, notification)
        }
    }

    private fun initBluetooth() {
        bluetoothManager = getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager
        bluetoothAdapter = bluetoothManager.adapter

        bluetoothReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context?, intent: Intent) {
                when (intent.action) {
                    ACTION_START_SCAN -> {
                        startScan()
                    }

                    ACTION_STOP_SCAN -> {
                        stopScan()
                    }
                }
            }
        }

        val intentFilter = IntentFilter()
        intentFilter.addAction(ACTION_START_SCAN)
        intentFilter.addAction(ACTION_STOP_SCAN)
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            registerReceiver(bluetoothReceiver, intentFilter, Context.RECEIVER_NOT_EXPORTED)
        }else{
            registerReceiver(bluetoothReceiver, intentFilter)
        }

        startScan()
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null
    }

    private fun startScan() {
        if (!bleScanning) {
            bleScanning = true
            bluetoothAdapter.bluetoothLeScanner.startScan(
                bleScanFilters, bleScanSettings, bleScanCallback
            )
        }
        //        delayClose()
    }

    private fun delayClose() {
        thread {
            Thread.sleep(5000)
            if (bleScanning) {
                stopScan()
            }
        }
    }

    private fun stopScan() {
        if (bleScanning) {
            bleScanning = false
            bluetoothAdapter.bluetoothLeScanner.stopScan(bleScanCallback)
        }
    }

    private fun sendResult() {
        val intent = Intent(ACTION_SCAN_RESULT)
        intent.putParcelableArrayListExtra(
            ACTION_SCAN_RESULT_DATA, ArrayList(deviceMap.keys)
        )
        sendBroadcast(intent)
    }

    override fun onDestroy() {
        unregisterReceiver(bluetoothReceiver)
        stopScan()
        super.onDestroy()
        Log.d("XDScanBLEService", "onDestroy: service destroyed")
    }
}